@@ -47,12 +47,12 @@ extern struct file_system_type cifs_fs_type;
 int cifsFYI = 0;
 int cifsERROR = 1;
 int traceSMB = 0;
-unsigned int oplockEnabled = 0;
+unsigned int oplockEnabled = 1;
 unsigned int lookupCacheEnabled = 1;
 unsigned int multiuser_mount = 0;
 unsigned int extended_security = 0;
 unsigned int ntlmv2_support = 0;
-unsigned int sign_CIFS_PDUs = 0;
+unsigned int sign_CIFS_PDUs = 1;
 unsigned int CIFSMaximumBufferSize = CIFS_MAX_MSGSIZE;
 struct task_struct * oplockThread = NULL;
 
@@ -223,7 +223,7 @@ cifs_show_options(struct seq_file *s, struct vfsmount *m)
 	if (cifs_sb) {
 		if (cifs_sb->tcon) {
 			seq_printf(s, ",unc=%s", cifs_sb->tcon->treeName);
-			if (cifs_sb->tcon->ses->userName)
+			if ((cifs_sb->tcon->ses) && (cifs_sb->tcon->ses->userName))
 				seq_printf(s, ",username=%s",
 					   cifs_sb->tcon->ses->userName);
 			if(cifs_sb->tcon->ses->domainName)
@@ -246,7 +246,9 @@ struct super_operations cifs_super_ops = {
 	.delete_inode	= cifs_delete_inode,  *//* Do not need the above two functions     
    unless later we add lazy close of inodes or unless the kernel forgets to call
    us with the same number of releases (closes) as opens */
+#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,17))
 	.show_options = cifs_show_options,
+#endif
 /*    .umount_begin   = cifs_umount_begin, *//* consider adding in the future */
 };
 
@@ -437,9 +439,8 @@ static int cifs_oplock_thread(void * dummyarg)
 	struct list_head * tmp;
 	struct list_head * tmp1;
 	struct oplock_q_entry * oplock_item;
-	struct file * pfile;
 	struct cifsTconInfo *pTcon;
-	int rc;
+	int rc = 0;
 
 	daemonize();
 	sprintf(current->comm,"cifsoplockd");
@@ -448,30 +449,28 @@ static int cifs_oplock_thread(void * dummyarg)
 		set_current_state(TASK_INTERRUPTIBLE);
 		schedule_timeout(100*HZ);
 		/* BB add missing code */
-		cFYI(1,("oplock thread woken up - flush inode")); /* BB remove */
 		write_lock(&GlobalMid_Lock); 
 		list_for_each_safe(tmp, tmp1, &GlobalOplock_Q) {
 			oplock_item = list_entry(tmp, struct oplock_q_entry,
 							       qhead);
 			if(oplock_item) {
 				pTcon = oplock_item->tcon;
-				pfile = oplock_item->file_to_flush;
-				cFYI(1,("process item on queue"));/* BB remove */
 				DeleteOplockQEntry(oplock_item);
 				write_unlock(&GlobalMid_Lock);
-				rc = filemap_fdatasync(pfile->f_dentry->d_inode->i_mapping);
+#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,17))
+				rc = filemap_fdatasync(oplock_item->pinode->i_mapping);
 				if(rc)
-					CIFS_I(pfile->f_dentry->d_inode)->write_behind_rc 
+					CIFS_I(oplock_item->pinode)->write_behind_rc 
 						= rc;
-				cFYI(1,("Oplock flush file %p rc %d",pfile,rc));
-				if(pfile->private_data) {
-					rc = CIFSSMBLock(0, pTcon, 
-						((struct cifsFileInfo *) pfile->private_data)->netfid,
-						0 /* len */ , 0 /* offset */, 0, 
-						0, LOCKING_ANDX_OPLOCK_RELEASE,
-						0 /* wait flag */);
-					cFYI(1,("Oplock release rc = %d ",rc));
-				}
+#else 
+				filemap_fdatasync(oplock_item->pinode->i_mapping);
+#endif
+				rc = CIFSSMBLock(0, pTcon, 
+					oplock_item->netfid,
+					0 /* len */ , 0 /* offset */, 0, 
+					0, LOCKING_ANDX_OPLOCK_RELEASE,
+					0 /* wait flag */);
+				cFYI(1,("Oplock release rc = %d ",rc));
 				write_lock(&GlobalMid_Lock);
 			} else
 				break;
