@@ -123,11 +123,12 @@ cifs_create(struct inode *inode, struct dentry *direntry, int mode)
 {
 	int rc = -ENOENT;
 	int xid;
-	int oplock = REQ_OPLOCK;
+	int oplock = 0;
 	__u16 fileHandle;
 	struct cifs_sb_info *cifs_sb;
 	struct cifsTconInfo *pTcon;
 	char *full_path = NULL;
+	FILE_ALL_INFO * buf = NULL;
 	struct inode *newinode = NULL;
 
 	xid = GetXid();
@@ -137,11 +138,14 @@ cifs_create(struct inode *inode, struct dentry *direntry, int mode)
 
 	full_path = build_path_from_dentry(direntry);
 
-	/* BB add processing for setting the equivalent of mode - e.g. via CreateX with ACLs */
+	/* BB add processing to set equivalent of mode - e.g. via CreateX with ACLs */
+	if (oplockEnabled)
+		oplock = REQ_OPLOCK;
 
-	rc = CIFSSMBOpen(xid, pTcon, full_path, FILE_OVERWRITE_IF, GENERIC_ALL
-			 /* 0x20197 was used previously */ , CREATE_NOT_DIR,
-			 &fileHandle, &oplock, cifs_sb->local_nls);
+        buf = kmalloc(sizeof(FILE_ALL_INFO),GFP_KERNEL);
+	rc = CIFSSMBOpen(xid, pTcon, full_path, FILE_OVERWRITE_IF,
+			GENERIC_ALL, CREATE_NOT_DIR,
+			&fileHandle, &oplock, buf, cifs_sb->local_nls);
 	if (rc) {
 		cFYI(1, ("cifs_create returned 0x%x ", rc));
 	} else {
@@ -150,34 +154,46 @@ cifs_create(struct inode *inode, struct dentry *direntry, int mode)
 						      inode->i_sb);
 		else
 			rc = cifs_get_inode_info(&newinode, full_path,
-						 inode->i_sb);
+						 buf, inode->i_sb);
 
 		if (rc != 0) {
 			cFYI(1,("Create worked but get_inode_info failed with rc = %d",
 			      rc));
-			/* close handle */
 		} else {
 			direntry->d_op = &cifs_dentry_ops;
 			d_instantiate(direntry, newinode);
 		}
-		/* BB check oplock state before deciding to call following */
-/*        if(*oplock)
-            save off handle in inode and dontdoclose */
-        
-		CIFSSMBClose(xid, pTcon, fileHandle);	
-        /* BB In the future chain close with the NTCreateX to narrow window */
-        
-        if(newinode)
-            newinode->i_mode = mode;
-	}
+
+                CIFSSMBClose(xid, pTcon, fileHandle);
+
+		if(newinode) {
+			newinode->i_mode = mode;
+			if (cifs_sb->tcon->ses->capabilities & CAP_UNIX)                
+				CIFSSMBUnixSetPerms(xid, pTcon, full_path, inode->i_mode,
+					(__u64)-1, 
+					(__u64)-1,
+					0 /* dev */,
+					cifs_sb->local_nls);
+			else { /* BB implement via Windows security descriptors */
+			/* eg CIFSSMBWinSetPerms(xid,pTcon,full_path,mode,-1,-1,local_nls);*/
+			/* in the meantime could set r/o dos attribute when perms are eg:
+				mode & 0222 == 0 */
+			}
+		}
+
+
+	} 
+
+	if (buf)
+	    kfree(buf);
 	if (full_path)
-		kfree(full_path);
+	    kfree(full_path);
 	FreeXid(xid);
 
 	return rc;
 }
 
-int cifs_mknod(struct inode *inode, struct dentry *direntry, int mode, dev_t device_number) 
+int cifs_mknod(struct inode *inode, struct dentry *direntry, int mode, int device_number) 
 {
 	int rc = -EPERM;
 	int xid;
@@ -248,7 +264,7 @@ cifs_lookup(struct inode *parent_dir_inode, struct dentry *direntry)
 		rc = cifs_get_inode_info_unix(&newInode, full_path,
 					      parent_dir_inode->i_sb);
 	else
-		rc = cifs_get_inode_info(&newInode, full_path,
+		rc = cifs_get_inode_info(&newInode, full_path, NULL,
 					 parent_dir_inode->i_sb);
 
 	if ((rc == 0) && (newInode != NULL)) {
