@@ -86,20 +86,21 @@ cifs_open(struct inode *inode, struct file *file)
 	if (file->f_flags & O_CREAT)
 		disposition = FILE_OVERWRITE;
 
-    /* BB first check if file has batch oplock (or oplock ?) */
-
-	/* BB finish adding in oplock support BB */
 	if (oplockEnabled)
 		oplock = REQ_OPLOCK;
 	else
 		oplock = FALSE;
 
 	/* BB pass O_SYNC flag through on file attributes .. BB */
+
+	/* BB add code to refresh inode by passing in file_info buf on open 
+	   and calling get_inode_info with returned buf (at least 
+	   helps non-Unix server case */
 	rc = CIFSSMBOpen(xid, pTcon, full_path, disposition, desiredAccess,
-			CREATE_NOT_DIR, &netfid, &oplock, cifs_sb->local_nls);
+			CREATE_NOT_DIR, &netfid, &oplock, NULL, cifs_sb->local_nls);
 	if (rc) {
 		cFYI(1, ("cifs_open returned 0x%x ", rc));
-		cFYI(1, ("oplock: %d ", oplock));
+		cFYI(1, ("oplock: %d ", oplock));	
 	} else {
 		file->private_data =
 		    kmalloc(sizeof (struct cifsFileInfo), GFP_KERNEL);
@@ -110,6 +111,7 @@ cifs_open(struct inode *inode, struct file *file)
 			pCifsFile->netfid = netfid;
 			pCifsFile->pid = current->pid;
 			pCifsFile->pfile = file; /* needed for writepage */
+			pCifsFile->pInode = inode;
 			spin_lock(&files_lock);
 			write_lock(&GlobalSMBSeslock);
 			list_add(&pCifsFile->tlist,&pTcon->openFileList);
@@ -164,12 +166,14 @@ int reopen_files(struct cifsTconInfo * pTcon, struct nls_table * nlsinfo)
 /* list all files open on tree connection */
 	read_lock(&GlobalSMBSeslock);
 	list_for_each_safe(tmp, tmp1, &pTcon->openFileList) {            
-		open_file = list_entry(tmp,struct cifsFileInfo, flist);
+		open_file = list_entry(tmp,struct cifsFileInfo, tlist);
 		if(open_file) {
 			if(open_file->search_resume_name) {
 				kfree(open_file->search_resume_name);
 			}
 			file = open_file->pfile;
+			list_del(&open_file->flist);
+			list_del(&open_file->tlist);
 			kfree(open_file);
 			if(file) {                
 				file->private_data = NULL;
@@ -206,8 +210,7 @@ cifs_close(struct inode *inode, struct file *file)
 	pTcon = cifs_sb->tcon;
 	if (pSMBFile) {
 		spin_lock(&files_lock);
-		if(pSMBFile->flist.next)
-			list_del(&pSMBFile->flist);
+		list_del(&pSMBFile->flist);
 		list_del(&pSMBFile->tlist);
 		spin_unlock(&files_lock);
 		rc = CIFSSMBClose(xid, pTcon, pSMBFile->netfid);
@@ -561,8 +564,13 @@ cifs_fsync(struct file *file, struct dentry *dentry, int datasync)
 
 	cFYI(1, ("Sync file - name: %s datasync: 0x%x ", 
 		dentry->d_name.name, datasync));
-	
+#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,17))	
 	rc = filemap_fdatasync(inode->i_mapping);
+#else
+	filemap_fdatasync(inode->i_mapping);
+#endif
+
+
 	if(rc == 0)
 		CIFS_I(inode)->write_behind_rc = 0;
 
@@ -615,8 +623,12 @@ int cifs_flush(struct file *file)
 	/* deal with writebehind errors */
 	/* unlock inode for writing */
 	/* filemapfdatawrite appears easier for the time being */
-
+#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,17))
 	rc = filemap_fdatasync(inode->i_mapping);
+#else
+	filemap_fdatasync(inode->i_mapping);
+#endif
+
 	cFYI(1,("Flush inode %p file %p rc %d",inode,file,rc));
 	if(rc == 0) /* reset wb rc if we were able to write out dirty pages */
 		CIFS_I(inode)->write_behind_rc = 0;
@@ -1245,7 +1257,7 @@ cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 					pfindDataUnix->FileName, 
 					cifsFile->resume_name_length);
 			}
-			for (i = 2; i < findParms.SearchCount + 2; i++) {
+			for (i = 2; i < (unsigned int)findParms.SearchCount + 2; i++) {
 				if (UnixSearch == FALSE) {
 					pfindData->FileNameLength =
 					  le32_to_cpu(pfindData->FileNameLength);
