@@ -174,24 +174,30 @@ cifs_get_inode_info_unix(struct inode **pinode,
 }
 
 int
-cifs_get_inode_info(struct inode **pinode,
-		    const unsigned char *search_path, struct super_block *sb)
+cifs_get_inode_info(struct inode **pinode, const unsigned char *search_path, 
+		FILE_ALL_INFO * pfindData, struct super_block *sb)
 {
 	int xid;
 	int rc = 0;
-	FILE_ALL_INFO findData;
 	struct cifsTconInfo *pTcon;
 	struct inode *inode;
 	struct cifs_sb_info *cifs_sb = CIFS_SB(sb);
 	char *tmp_path;
+	char *buf = NULL;
 
 	xid = GetXid();
 
 	pTcon = cifs_sb->tcon;
 	cFYI(1, (" Getting info on %s ", search_path));
-	/* we could have done a find first instead but this returns more info */
-	rc = CIFSSMBQPathInfo(xid, pTcon, search_path, &findData,
+
+	/* if file info not passed in then get it from server */
+	if(pfindData == NULL) {
+		buf = kmalloc(sizeof(FILE_ALL_INFO),GFP_KERNEL);
+		pfindData = (FILE_ALL_INFO *)buf;
+	/* could do find first instead but this returns more info */
+		rc = CIFSSMBQPathInfo(xid, pTcon, search_path, pfindData,
 			      cifs_sb->local_nls);
+	}
 	/* dump_mem("\nQPathInfo return data",&findData, sizeof(findData)); */
 	if (rc) {
 		if (rc == -EREMOTE) {
@@ -202,8 +208,10 @@ cifs_get_inode_info(struct inode **pinode,
 				    strnlen(search_path, MAX_PATHCONF) + 1,
 				    GFP_KERNEL);
 			if (tmp_path == NULL) {
-				FreeXid(xid);
-				return -ENOMEM;
+			    if(buf)
+				kfree(buf);
+			    FreeXid(xid);
+			    return -ENOMEM;
 			}
 
 			strncpy(tmp_path, pTcon->treeName, MAX_TREE_SIZE);
@@ -214,8 +222,10 @@ cifs_get_inode_info(struct inode **pinode,
 			kfree(tmp_path);
 			/* BB fix up inode etc. */
 		} else if (rc) {
-			FreeXid(xid);
-			return rc;
+		    if(buf)
+			kfree(buf);
+		    FreeXid(xid);
+		    return rc;
 		}
 	} else {
 		struct cifsInodeInfo *cifsInfo;
@@ -227,8 +237,8 @@ cifs_get_inode_info(struct inode **pinode,
 
 		inode = *pinode;
 		cifsInfo = CIFS_I(inode);
-		findData.Attributes = le32_to_cpu(findData.Attributes);
-		cifsInfo->cifsAttrs = findData.Attributes;
+		pfindData->Attributes = le32_to_cpu(pfindData->Attributes);
+		cifsInfo->cifsAttrs = pfindData->Attributes;
 		cFYI(1, (" Old time %ld ", cifsInfo->time));
 		cifsInfo->time = jiffies;
 		cFYI(1, (" New time %ld ", cifsInfo->time));
@@ -238,21 +248,21 @@ cifs_get_inode_info(struct inode **pinode,
 		    (pTcon->ses->server->maxBuf - MAX_CIFS_HDR_SIZE) & 0xFFFFFE00;
 		/* Linux can not store file creation time unfortunately so we ignore it */
 		inode->i_atime =
-		    cifs_NTtimeToUnix(le64_to_cpu(findData.LastAccessTime));
+		    cifs_NTtimeToUnix(le64_to_cpu(pfindData->LastAccessTime));
 		inode->i_mtime =
-		    cifs_NTtimeToUnix(le64_to_cpu(findData.LastWriteTime));
+		    cifs_NTtimeToUnix(le64_to_cpu(pfindData->LastWriteTime));
 		inode->i_ctime =
-		    cifs_NTtimeToUnix(le64_to_cpu(findData.ChangeTime));
+		    cifs_NTtimeToUnix(le64_to_cpu(pfindData->ChangeTime));
 		cFYI(0,
-		     (" Attributes came in as 0x%x ", findData.Attributes));
+		     (" Attributes came in as 0x%x ", pfindData->Attributes));
 
 		/* set default mode. will override for dirs below */
 		inode->i_mode = cifs_sb->mnt_file_mode;
 
-		if (findData.Attributes & ATTR_REPARSE) {
+		if (pfindData->Attributes & ATTR_REPARSE) {
 	/* Can IFLNK be set as it basically is on windows with IFREG or IFDIR? */
 			inode->i_mode |= S_IFLNK;
-		} else if (findData.Attributes & ATTR_DIRECTORY) {
+		} else if (pfindData->Attributes & ATTR_DIRECTORY) {
 	/* override default perms since we do not do byte range locking on dirs */
 			inode->i_mode = cifs_sb->mnt_dir_mode;
 			inode->i_mode |= S_IFDIR;
@@ -263,14 +273,14 @@ cifs_get_inode_info(struct inode **pinode,
 				inode->i_mode &= ~(S_IWUGO);
    /* BB add code here - validate if device or weird share or device type? */
 		}
-		inode->i_size = le64_to_cpu(findData.EndOfFile);
-		findData.AllocationSize = le64_to_cpu(findData.AllocationSize);
+		inode->i_size = le64_to_cpu(pfindData->EndOfFile);
+		pfindData->AllocationSize = le64_to_cpu(pfindData->AllocationSize);
 		inode->i_blocks =
-		    do_div(findData.AllocationSize, inode->i_blksize);
+		    do_div(pfindData->AllocationSize, inode->i_blksize);
 		cFYI(1,
 		     (" Size %ld and blocks %ld ",
 		      (unsigned long) inode->i_size, inode->i_blocks));
-		inode->i_nlink = le32_to_cpu(findData.NumberOfLinks);
+		inode->i_nlink = le32_to_cpu(pfindData->NumberOfLinks);
 
 		/* BB fill in uid and gid here? with help from winbind? 
 			or retrieve from NTFS stream extended attribute */
@@ -294,6 +304,8 @@ cifs_get_inode_info(struct inode **pinode,
 					   kdev_t_to_nr(inode->i_rdev));
 		}
 	}
+	if(buf)
+	    kfree(buf);
 	FreeXid(xid);
 	return rc;
 }
@@ -309,7 +321,7 @@ cifs_read_inode(struct inode *inode)
 	if (cifs_sb->tcon->ses->capabilities & CAP_UNIX)
 		cifs_get_inode_info_unix(&inode, "", inode->i_sb);
 	else
-		cifs_get_inode_info(&inode, "", inode->i_sb);
+		cifs_get_inode_info(&inode, "", NULL, inode->i_sb);
 }
 
 int
@@ -342,7 +354,7 @@ cifs_unlink(struct inode *inode, struct dentry *direntry)
 
 		rc = CIFSSMBOpen(xid, pTcon, full_path, FILE_OPEN, DELETE, 
 				CREATE_NOT_DIR | CREATE_DELETE_ON_CLOSE,
-				&netfid, &oplock, cifs_sb->local_nls);
+				&netfid, &oplock, NULL, cifs_sb->local_nls);
 		if(rc==0) {
 			CIFSSMBClose(xid, pTcon, netfid);
 			/* BB In the future chain close with the NTCreateX to narrow window */
@@ -406,12 +418,13 @@ cifs_mkdir(struct inode *inode, struct dentry *direntry, int mode)
 			rc = cifs_get_inode_info_unix(&newinode, full_path,
 						      inode->i_sb);
 		else
-			rc = cifs_get_inode_info(&newinode, full_path,
+			rc = cifs_get_inode_info(&newinode, full_path,NULL,
 						 inode->i_sb);
 
 		direntry->d_op = &cifs_dentry_ops;
 		d_instantiate(direntry, newinode);
-		direntry->d_inode->i_nlink = 2;
+		if(direntry->d_inode)
+			direntry->d_inode->i_nlink = 2;
 		if (cifs_sb->tcon->ses->capabilities & CAP_UNIX)                
 			CIFSSMBUnixSetPerms(xid, pTcon, full_path, mode,
 				(__u64)-1,  
@@ -550,7 +563,7 @@ cifs_revalidate(struct dentry *direntry)
 		cifs_get_inode_info_unix(&direntry->d_inode, full_path,
 					 direntry->d_sb);
 	else
-		cifs_get_inode_info(&direntry->d_inode, full_path,
+		cifs_get_inode_info(&direntry->d_inode, full_path, NULL,
 				    direntry->d_sb);
 
 	/* BB if not oplocked, invalidate inode pages if mtime has changed */
